<!--htmlhint id-unique:false -->

<div ng-if="bare" class="iframe-wrapper">
  <div id="iframe-wrapper-{{id}}">
  </div>
</div>

<section ng-if="!bare" class="section-panel">
  <div class="container-fluid">
    <div class="row section-header">
      <div class="col-lg-12 col-md-12 col-sm-12">
        <h3>
          {{ title }}
          <a href="{{externalUrl || iframeUrl}}" target="_blank" class="btn">
            Open in a new window
          </a>
        </h3>
      </div>
    </div>
    <div class="row padded">
      <div class="col-lg-12 col-md-12 col-sm-12">
        {{description}}
        <div class="iframe-wrapper">
          <div id="iframe-wrapper-{{id}}">
          </div>
        </div>
      </div>
    </div>
  </div>
</section>
