{% extends 'orchestra/base.html' %}
{% load staticfiles %}

{% block title %}Dashboard{% endblock %}

{% block page_stylesheets %}
    <!-- TODO(jrbotros): remove other scripts once they've been bundled properly -->
    <link rel="stylesheet" href="{% static "dist/main.css" %}">

    {# we call `static()` in the view function to prevent whitenoise looking for a template value #}
    {% for css in stylesheet_includes %}
      {{ css | safe }}
    {% endfor %}
{% endblock page_stylesheets%}

{% block main %}
<section id="main-content" ng-view autoscroll="true">
  {% block content %}content{% endblock content %}
</section>
{% endblock %}

{% block page_javascript %}
<script type="text/javascript">
    window.orchestra = {{ orchestra_arguments | safe }};
</script>

<script src="https://ajax.googleapis.com/ajax/libs/angularjs/1.5.7/angular.min.js"></script>
<script src="https://ajax.googleapis.com/ajax/libs/angularjs/1.5.7/angular-route.min.js"></script>
<script src="//ajax.googleapis.com/ajax/libs/angularjs/1.5.5/angular-sanitize.min.js"></script>
<script src="//cdnjs.cloudflare.com/ajax/libs/angular-ui-bootstrap/1.3.1/ui-bootstrap-tpls.min.js"></script>
<script src="//cdnjs.cloudflare.com/ajax/libs/angular-ui-select/0.16.1/select.min.js" type="text/javascript"></script>

<!-- TODO(jrbotros): remove other scripts once they've been bundled properly -->
<script src="{% static "dist/main.js" %}" type="text/javascript"></script>

{% for js in javascript_includes %}
  {# we call `static()` in the view function to prevent whitenoise looking for a template value #}
  {{ js | safe }}
{% endfor %}
{% endblock page_javascript%}
