# -*- coding: utf-8 -*-
# Generated by Django 1.11.12 on 2018-06-29 18:27
from __future__ import unicode_literals

from django.db import migrations, models
import jsonfield.fields


class Migration(migrations.Migration):

    dependencies = [
        ('orchestra', '0076_auto_20180615_1606'),
    ]

    operations = [
        migrations.AddField(
            model_name='todolisttemplate',
            name='conditional_property_function',
            field=jsonfield.fields.JSONField(default={}),
        ),
        migrations.AlterField(
            model_name='todo',
            name='description',
            field=models.TextField(),
        ),
    ]
