# -*- coding: utf-8 -*-
# Generated by Django 1.9.5 on 2016-04-11 15:10
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('orchestra', '0035_create_time_entries_for_snapshots'),
    ]

    operations = [
        migrations.RemoveField(  # manually-reviewed
            model_name='taskassignment',
            name='snapshots',
        ),
    ]
