# -*- coding: utf-8 -*-
# Generated by Django 1.9.13 on 2017-10-25 08:37
from __future__ import unicode_literals

from django.db import migrations
from django.db import models


class Migration(migrations.Migration):

    dependencies = [
        ('orchestra', '0070_todos_and_reverse_assignments'),
    ]

    operations = [
        migrations.AddField(
            model_name='workercertification',
            name='staffing_priority',
            field=models.IntegerField(default=0),
        ),
    ]
