{% extends 'orchestra/base.html' %}
{% load staticfiles %}

{% block title %}
  {% if response.is_winner %}
    Success!
  {% else %}
    Task Taken
  {% endif %}
{% endblock %}

{% block main %}
<div class="container-fluid main-content communication-container">
  <div class="col-lg-6 col-lg-offset-3 col-md-8 col-md-offset-2 col-sm-10 col-sm-offset-1 col-xs-12">
    <div class="card">
        {% if response.is_winner %}
        <h2>
          Congratulations! You got the task :)
        </h2>
          <div class="row spaced">
            <a href="{% url "orchestra:index" %}/task/{{ response.request_inquiry.request.task.id }}/" class="btn btn-primary btn-lg">
              View Task
            </a>
          </div>
        {% else %}
          <h2>
            Unfortunately, task has already been taken :(
          </h2>
          <div class="row spaced">
            <a href="{% url "orchestra:communication:available_staffing_requests" %}" class="btn btn-primary btn-lg">
              View All Available Tasks
            </a>
          </div>
        {% endif %}
    </div>
  </div>
</div>
{% endblock %}
